/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright© 2020 MikroElektronika d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
 * OR OTHER DEALINGS IN THE SOFTWARE. 
 */

/*!
 * \file
 *
 * \brief This file contains API for MCP2542 Click driver.
 *
 * \addtogroup mcp2542 MCP2542 Click Driver
 * @{
 */
// ----------------------------------------------------------------------------

#ifndef MCP2542_H
#define MCP2542_H

/**
 * Any initialization code needed for MCU to function properly.
 * Do not remove this line or clock might not be set correctly.
 */
#ifdef PREINIT_SUPPORTED
#include "preinit.h"
#endif

#ifdef MikroCCoreVersion
    #if MikroCCoreVersion >= 1
        #include "delays.h"
    #endif
#endif

#include "drv_digital_out.h"
#include "drv_digital_in.h"
#include "drv_uart.h"


// -------------------------------------------------------------- PUBLIC MACROS 
/**
 * \defgroup macros Macros
 * \{
 */

/**
 * \defgroup map_mikrobus MikroBUS
 * \{
 */
#define MCP2542_MAP_MIKROBUS( cfg, mikrobus ) \
  cfg.tx_pin  = MIKROBUS( mikrobus, MIKROBUS_TX ); \
  cfg.rx_pin  = MIKROBUS( mikrobus, MIKROBUS_RX ); \
  cfg.stb = MIKROBUS( mikrobus, MIKROBUS_AN )
/** \} */

/**
 * \defgroup driver Driver define
 * \{
 */
#define MCP2542_DRV_RX_BUFFER_SIZE 500
#define MCP2542_DRV_TX_BUFFER_SIZE 500
/** \} */  

/**
 * \defgroup error_code Error Code
 * \{
 */
#define MCP2542_RETVAL  uint8_t

#define MCP2542_OK           0x00
#define MCP2542_INIT_ERROR   0xFF
/** \} */

/**
 * \defgroup standby_mode Standby mode
 * \{
 */
#define MCP2542_STANDBY_MODE_ON           1
#define MCP2542_STANDBY_MODE_OFF          0
/** \} */

/** \} */ // End group macro 
// --------------------------------------------------------------- PUBLIC TYPES
/**
 * \defgroup type Types
 * \{
 */

//typedef  uart_data_t char;

/**
 * @brief Click ctx object definition.
 */
typedef struct
{
    // Output pins 

    digital_out_t stb;

    uart_t uart;

    uint8_t uart_rx_buffer[ MCP2542_DRV_RX_BUFFER_SIZE ];
    uint8_t uart_tx_buffer[ MCP2542_DRV_TX_BUFFER_SIZE ];

} mcp2542_t;

/**
 * @brief Click configuration structure definition.
 */
typedef struct
{
    // Communication gpio pins 

    pin_name_t rx_pin;
    pin_name_t tx_pin;
    
    // Additional gpio pins 

    pin_name_t stb;


    // static variable 

    uint32_t          baud_rate;     // Clock speed.
    bool              uart_blocking;
    uart_data_bits_t  data_bit;      // Data bits.
    uart_parity_t     parity_bit;    // Parity bit.  
    uart_stop_bits_t  stop_bit;      // Stop bits.    

} mcp2542_cfg_t;

/** \} */ // End types group
// ----------------------------------------------- PUBLIC FUNCTION DECLARATIONS

/**
 * \defgroup public_function Public function
 * \{
 */
 
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief Config Object Initialization function.
 *
 * @param cfg  Click configuration structure.
 *
 * @description This function initializes click configuration structure to init state.
 * @note All used pins will be set to unconnected state.
 */
void mcp2542_cfg_setup ( mcp2542_cfg_t *cfg );

/**
 * @brief Initialization function.
 * @param mcp2542 Click object.
 * @param cfg Click configuration structure.
 * 
 * @description This function initializes all necessary pins and peripherals used for this click.
 */
MCP2542_RETVAL mcp2542_init ( mcp2542_t *ctx, mcp2542_cfg_t *cfg );

/**
 * @brief Click Default Configuration function.
 *
 * @param ctx          Click object.
 *
 * @description This function executes default configuration for USB_UART click.
 */
void mcp2542_default_cfg( mcp2542_t *ctx );

/**
 * @brief Generic multi write function.
 *
 * @param ctx          Click object.
 * @param data_buf     Input data buffer
 * @param len          Number of the bytes in the data buffer
 *
 * @description This function writes data.
 */
void mcp2542_generic_multi_write ( mcp2542_t *ctx, uint8_t *data_buf,  uint8_t len );

/**
 * @brief Generic multi read function.
 *
 * @param ctx        Click object.
 * @param data_buf   Output data buffer.
 * @param len        Number of the bytes in data buffer.
 *
 * @description This function reads data.
 */
void mcp2542_generic_multi_read ( mcp2542_t *ctx, uint8_t *data_buf,  uint8_t len );


/**
 * @brief Generic single read function.
 *
 * @param ctx        Click object.
 * @return one byte data
 *
 * @description This function read one byte data.
 */
uint8_t mcp2542_generic_single_read ( mcp2542_t *ctx );

/**
 * @brief Generic single write function.
 *
 * @param ctx          Click object.
 * @param tx_data      Input data 
 *
 * @description This function write one byte data.
 */
void mcp2542_generic_single_write ( mcp2542_t *ctx, uint8_t tx_data );

/**
 * @brief Set device mode.
 *
 * @param ctx          Click object.
 * @param mode         Mode select.
 *
 * @description This function set device mode.
 */
void mcp2542_set_mode ( mcp2542_t *ctx, uint8_t mode );

#ifdef __cplusplus
}
#endif
#endif  // _MCP2542_H_

/** \} */ // End public_function group
/// \}    // End click Driver group  
/*! @} */
// ------------------------------------------------------------------------- END
